# 1. Vector

## 1) Declaration

You can declare Vector by several ways.

1. Vector [1 .. 10]
2. vector [1 .. 10]
3. vec [1.. 10]
4. fromList [1 .. 10]

For convenience, `vec` is easy way to declare Vector.

## 2) Functor

Vector has Functor instance.

```haskell
instance Functor Vector where
  fmap f (Vector x) = Vector (fmap f x)
```

So, it's easy to apply function to each component of Vector.

For example,

```haskell
(+1) <$> vec [1,2,3] -- Vector [2,3,4]
(*2) <$> vec [1,2,3] -- Vector [2,4,6]
(^2) <$> vec [1,2,3] -- Vector [1,4,9]
```

## 3) Applicative

Vector has Applicative instance.

```haskell
instance Applicative Vector where
  pure a                  = Vector []
  Vector fs <*> Vector xs = Vector (zipWith ($) fs xs)
```

So, we can curry with Vector. For example,

```haskell
(+) <$> vec [1,2,3] <*> vec [3,2,1] -- Vector [4,4,4]
```

## 4) Num, Fractional, Floating

Since we have Functor, Applicative instances, we can implement most vectorized operators. But it's too bother thing to use functor or applicative operator. Thus, we need Num, Fractional, Floating instances.

For example to use,

```haskell
let a = vec [1,2,3]
let b = vec [3,2,1]

a + b -- Vector [4,4,4]
a - b -- Vector [-2,0,2]
a * b -- Vector [3,4,3]
a / b
abs a
sin a
cos a
tan a
```

## 5) Foldable

Since Vector is wrapper for list, we need length and fold operator.

For example,

```haskell
let a = vec [1,2,3]

length a  -- 3
sum a     -- 6
product a -- 6
maximum a -- 3
minimum a -- 1
```
